
using System;
using System.Collections;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Threading;
using ClarionSimulation;
using WorldServerLibrary;
using WorldServerLibrary.Model;
using WorldServerLibrary.Exceptions;

namespace ClarionDEMO
{
	class MainClass
	{
		#region properties

		public Thread thingsInWorld;
		private WorldServer worldServer = null;
		private ClarionAgent agent;
		String creatureId = String.Empty;
		String creatureName = String.Empty;
		double prad = 0;
		#endregion

		#region constructor
		public MainClass() {
			Console.WriteLine ("Clarion Demo V0.5");
			agent = new ClarionAgent();
			agent.OnNewVisualSensorialInformation += new InputVisualSensorialInformationEventHandler(agent_OnNewVisualSensorialInformation);
			agent.OnNewExternalActionSelected += new OutputActionChunkEventHandler(agent_OnNewExternalActionSelected);
			try
			{
				worldServer = new WorldServer("localhost", 4011);
				String message = worldServer.Connect();

				if (worldServer != null && worldServer.IsConnected)
				{
					Console.Out.WriteLine ("[SUCCESS] " + message + "\n");
					worldServer.SendWorldReset();
					worldServer.NewCreature(400, 200, 0, out creatureId, out creatureName);
					worldServer.SendCreateLeaflet();
					worldServer.NewBrick(4, 747, 2, 800, 567);
					worldServer.NewBrick(4, 50, -4, 747, 47);
					worldServer.NewBrick(4, 49, 562, 796, 599);
					worldServer.NewBrick(4, -2, 6, 50, 599);

					if (!String.IsNullOrWhiteSpace(creatureId))
					{
						worldServer.SendStartCamera(creatureId);
						worldServer.SendStartCreature(creatureId);
					}

					Console.Out.WriteLine("Creature created with name: " + creatureId + "\n");
					agent.Run();
					Console.Out.WriteLine("Running Simulation ...\n");
					thingsInWorld = new Thread(Run);
					thingsInWorld.Start();
				}
			}
			catch (WorldServerInvalidArgument invalidArtgument)
			{
				thingsInWorld.Abort ();
				Console.Out.WriteLine(String.Format("[ERROR] Invalid Argument: {0}\n", invalidArtgument.Message));
			}
			catch (WorldServerConnectionError serverError)
			{
				thingsInWorld.Abort ();
				Console.Out.WriteLine(String.Format("[ERROR] Is is not possible to connect to server: {0}\n", serverError.Message));
			}
			catch (Exception ex)
			{
				thingsInWorld.Abort ();
				Console.Out.WriteLine(String.Format("[ERROR] Unknown Error: {0}\n", ex.Message));
			}
		}
		#endregion

		#region Methods
		public static void Main (string[] args)	{
			new MainClass();
		}

		private void Run(object obj){
			while (true) {
				this.generateJewels ();
				this.generateFoods ();
				Thread.Sleep (30000);
			}
		}

		private void generateJewels(){
			Random random = new Random ();
			int count = random.Next (5, 15);

			for (int i=0; i < count; i++) {
				int posX = random.Next (100, 700);
				int posY = random.Next (100, 500);
				worldServer.NewJewel (random.Next (0, 6), posX, posY);
			}

		}

		private void generateFoods(){
			Random random = new Random ();
			int count = random.Next (1, 5);

			for (int i=0; i < count; i++) {
				int posX = random.Next (100, 700);
				int posY = random.Next (100, 500);
				worldServer.NewFood(1, posX, posY);
			}

		}

		IList<Thing> agent_OnNewVisualSensorialInformation()
		{
			IList<Thing> response = null;

			if (worldServer != null && worldServer.IsConnected)
			{
				response = worldServer.SendGetCreatureState(creatureName);
				prad = (Math.PI / 180) * response.First().Pitch;
				while (prad > Math.PI) prad -= 2 * Math.PI;
				while (prad < - Math.PI) prad += 2 * Math.PI;
				Sack s = worldServer.SendGetSack("0");
				this.agent.sack = s;
				s.print();

			}

			return response;
		}

		void agent_OnNewExternalActionSelected(ClarionAgentActionType externalAction)
		{   Thread.CurrentThread.CurrentCulture = new CultureInfo("en-US");
			if (worldServer != null && worldServer.IsConnected)
			{
				switch (externalAction)
				{
					case ClarionAgentActionType.DO_NOTHING:
						agent.Abort (true);
					break;
					case ClarionAgentActionType.ROTATE:
						double newprad = prad + Math.PI/1000;
						while (newprad > Math.PI) newprad -= 2 * Math.PI;
						while (newprad < - Math.PI) newprad += 2 * Math.PI;
						worldServer.SendSetAngle(creatureId, 0, 0, newprad);
					break;

					case ClarionAgentActionType.MOVE_THING:
						worldServer.SendSetGoTo(creatureId, 1, 1, agent.getCurrentThing().X1, agent.getCurrentThing().Y1);
					break;

					case ClarionAgentActionType.GET_THING:
						if(agent.getCurrentThing().CategoryId == Thing.CATEGORY_JEWEL)
							worldServer.SendSackIt(creatureId, agent.getCurrentThing().Name);
						else
							worldServer.SendEatIt(creatureId, agent.getCurrentThing().Name);
					break;

					default:
					break;
				}
			}
		}

		#endregion
	}


}
